@@ -33,7 +33,7 @@
 #include "gi/cwrapper.h"
 #include "gjs/atoms.h"
 #include "gjs/context-private.h"
-#include "gjs/jsapi-class.h"  // IWYU pragma: keep
+#include "gjs/jsapi-class.h"
 #include "gjs/jsapi-util.h"
 #include "gjs/macros.h"
 #include "gjs/profiler-private.h"
