@@ -210,3 +210,38 @@ describe('Complete enumeration of GIRepositoryNamespace (new_enumerate)', functi
         }).not.toThrowError(/API of type .* not implemented, cannot define .*/);
     });
 });
+
+describe('Backwards compatibility for GLib/Gio platform specific GIRs', function () {
+    // Only test this if GioUnix is available
+    const skip = imports.gi.versions.GioUnix !== '2.0';
+
+    it('GioUnix objects are looked up in GioUnix, not Gio', function () {
+        if (skip) {
+            pending('GioUnix required for this test');
+            return;
+        }
+
+        GLib.test_expect_message('Gjs', GLib.LogLevelFlags.LEVEL_WARNING,
+            '*Gio.UnixMountMonitor*');
+
+        const monitor = Gio.UnixMountMonitor.get();
+        expect(monitor.toString()).toContain('GIName:GioUnix.MountMonitor');
+
+        GLib.test_assert_expected_messages_internal('Gjs',
+            'testIntrospection.js', 0,
+            'Expected deprecation message for Gio.Unix -> GioUnix');
+    });
+
+    it('has some exceptions', function () {
+        expect(Gio.UnixConnection.toString()).toContain('Gio_UnixConnection');
+
+        const credentialsMessage = new Gio.UnixCredentialsMessage();
+        expect(credentialsMessage.toString()).toContain('GIName:Gio.UnixCredentials');
+
+        const fdList = new Gio.UnixFDList();
+        expect(fdList.toString()).toContain('GIName:Gio.UnixFDList');
+
+        const socketAddress = Gio.UnixSocketAddress.new_with_type('', Gio.UnixSocketAddressType.ANONYMOUS);
+        expect(socketAddress.toString()).toContain('GIName:Gio.UnixSocketAddress');
+    });
+});
