@@ -8,12 +8,15 @@
 
 #include <config.h>
 
-#include <glib.h>    // for g_assert_...
 #include <stdint.h>  // for uintptr_t
+
+#include <iterator>  // for pair
 #include <limits>    // for numeric_limits
 #include <string>
 #include <type_traits>  // for is_same
-#include <utility>
+#include <utility>      // IWYU pragma: keep
+
+#include <glib.h>  // for g_assert_...
 
 #include "gjs/context.h"
 
